# Author(s): Kristo Mela
# This source code is licensed under the MIT license. See LICENSE in the repository root directory.
# Copyright 2022 Kristo Mela
# -*- coding: utf-8 -*-
import unittest

#from .framefem import FrameFEM, BeamSection, LineLoad, PointLoad
#from .elements import EBBeam

import numpy as np
import metku.framefem as ff
from metku.framefem.elements.ebbeam import EBBeam, EBBeam3D
from metku.framefem.elements.springs import LinearSpring
from metku.sections.steel.ISection import IPE, HEA

class TestFrameFEM(unittest.TestCase):
    def test_adding_material(self):
        fem = FrameFEM()
        fem.add_material(100, 200, 300)
        mat = fem.materials[0]
        self.assertEqual(100, mat.young)
        self.assertEqual(200, mat.nu)
        self.assertEqual(300, mat.density)

    def test_adding_node(self):
        fem = FrameFEM()
        fem.add_node(0, 0)
        node = fem.nodes[0]
        self.assertEqual([0, 0], [node.x, node.y])

    def test_linear_statics(self):
        fem = FrameFEM()
        # Nodes
        fem.add_node(0, 0)
        fem.add_node(0, 1)
        fem.add_node(1, 1)
        fem.add_node(1, 0)
        # Material
        fem.add_material(210e3, 0.3, 7850e-9)
        # Supports
        fem.add_support(1, 0, [0, 1, 2])
        fem.add_support(1, 3, [0, 1, 2])
        # Sections
        sect = BeamSection(1e3, 2e6)
        fem.add_section(sect)
        # Elements
        for nid in range(fem.nnodes() - 1):
            n1 = fem.nodes[nid]
            n2 = fem.nodes[nid + 1]
            mat = fem.materials[0]
            sect = fem.sections[0]
            ele = EBBeam(n1, n2, sect, mat)
            fem.add_element(ele)
        # Loads
        pointload = PointLoad(1, fem.nodes[1], [10, 0, 0], f=1)
        lineload = LineLoad(1, fem.elements[1], [0, 1], [-10, -10], 1)
        fem.add_load(pointload)
        fem.add_load(lineload)
        # Loadcase
        fem.add_loadcase(1, 1)
        # Linear statics
        fem.nodal_dofs()
        fem.linear_statics()
        

def simple_portal_fem(L,H,ne):
    """ Yksinkertainen portaalikehä """
    pf = ff.FrameFEM()
    
    col_sect = IPE(220)
    beam_sect = IPE(220)
    
    #col = ff.BeamSection(col_sect.A,col_sect.Iy)
    #beam = ff.BeamSection(beam_sect.A,beam_sect.Iy)
    
    pf.add_section(col_sect)
    pf.add_section(beam_sect)
    
    pf.add_material(col_sect.E,col_sect.material.nu,col_sect.density)
    # Generoidaan solmut
    
    Y = np.linspace(0,H,ne+1)
    X = np.linspace(0,L,ne+1)
    
    for y in Y[:-1]:
        pf.add_node(X[0],y)
    
    for x in X[:-1]:
        pf.add_node(x,Y[-1])
    
    for y in reversed(Y):
        pf.add_node(X[-1],y)
    
    for (k,node) in enumerate(pf.nodes[:-1]):
        if (node.x == 0 and pf.nodes[k+1].x == 0) or (node.x == L and pf.nodes[k+1].x == L):
                el_sec = col_sect
        else:
            el_sec = beam_sect
            
            
        pf.add_element(EBBeam(node,pf.nodes[k+1],el_sec,pf.materials[0]))
        
    pf.add_support(sid=0,nid=0,dof=[0,1,2],val=0.0)
    pf.add_support(sid=0,nid=pf.nnodes()-1,dof=[0,1,2],val=0.0)
    
    
    #F = -200e3
    F = -5e3
    
    pl1 = ff.PointLoad(sid=1,node=pf.nodes[ne],v=[F,0,0],f=1.0)
    pl2 = ff.PointLoad(sid=1,node=pf.nodes[2*ne],v=[F,0,0],f=1.0)
    pf.add_load(pl1)
    pf.add_load(pl2)
    
    for i in range(ne,2*ne):
        #print(i)
        pf.add_load(ff.LineLoad(sid=1,eid=pf.elements[i],xloc=[0,1],qval=[-20,-20],direction='y'))
    #pf.add_load(ll1)    
    
    #pl3 = ff.PointLoad(sid=1,node=pf.nodes[ne],v=[-F,0,0],f=1.0)
    #pf.add_load(pl3)
    
    pf.add_loadcase(supp_id=0,load_id=1)
    
    pf.nodal_dofs()
    
    KG = []
    
    pf.linear_statics(lcase=1,support_method="REM")
    #(w,v,KG) = pf.linear_buckling(k=10)
    #print(w)
    #nd = np.argmin(abs(pf.load_factors))
    #nd_sort = np.argsort(abs(pf.load_factors))
    #pf.draw(buckling_mode=nd_sort[0],scale=40000.0)
    pf.draw(deformed=1,scale=50)
    
    return pf, KG

def simple_column(L,ne=6):
    """ Mastopilari """
    pf = ff.FrameFEM()
    
    col_sect = IPE(220)
    
    pf.add_section(col_sect)
    
    pf.add_material(col_sect.E,col_sect.material.nu,col_sect.density)
    # Generoidaan solmut
    
    Y = np.linspace(0,L,ne+1)
    
    for y in Y:
        pf.add_node(0,y)
    
    for (k,node) in enumerate(pf.nodes[:-1]):                    
        pf.add_element(EBBeam(node,pf.nodes[k+1],col_sect,pf.materials[0]))
        
    pf.add_support(sid=0,nid=0,dof=[0,1,2],val=0.0)
    
    F = -200e3
    
    pl1 = ff.PointLoad(sid=1,node=pf.nodes[ne],v=[0,F,0],f=1.0)
    pf.add_load(pl1)
    
    pf.add_loadcase(supp_id=0,load_id=1)
    
    pf.nodal_dofs()
    pf.linear_statics(support_method="REM")
    """
    (w,v, KG) = pf.linear_buckling(k=3)
    #print(w)
    nd_sort = np.argsort(abs(pf.load_factors))
    nd = np.argmin(abs(pf.load_factors))
    print(nd)
    pf.draw(buckling_mode=nd_sort[0],scale=1.0)
    """
    return pf #, KG
    
def test_hinge():
    
    a = 5000
    b = 2000
    
    f = ff.FrameFEM()
    
    sect = IPE(200)
    
    f.add_section(sect)
    
    f.add_material(sect.E,sect.material.nu,sect.density)
    
    f.add_node(0,0)
    f.add_node(a,0)
    f.add_node(a+b,0)
    
    f.add_element(EBBeam(f.nodes[0],f.nodes[1],sect,f.materials[0]))
    f.add_element(EBBeam(f.nodes[1],f.nodes[2],sect,f.materials[0]))

    f.add_support(sid=0,nid=0,dof=[0,1,2],val=0.0)
    f.add_support(sid=0,nid=2,dof=[0,1,2],val=0.0)
    
    F = -200e3
    
    pl1 = ff.PointLoad(sid=1,node=f.nodes[1],v=[0,F,0],f=1.0)
    f.add_load(pl1)
    
    ll1 = ff.LineLoad(sid=1,eid=f.elements[0],xloc=[0,1],qval=[-10,-10],direction='y')
    #f.add_load(ll1)    
    f.add_loadcase(supp_id=0,load_id=1)
    
    # Vapautetaan momentti
    f.add_release(0,[5])
    #f.add_release(1,[2])
    
    f.nodal_dofs()
    f.linear_statics(lcase=1,support_method="REM")

    f.draw()
    
    return f

def test_springs():
    
    f = ff.FrameFEM()
    
    a = 1.0
    
    f.add_node(0,0)
    f.add_node(a,0)
    f.add_node(2*a,0)
    f.add_node(3*a,0)
    
    f.add_element(LinearSpring(f.nodes[0],f.nodes[1],k=1000))
    f.add_element(LinearSpring(f.nodes[1],f.nodes[2],k=2000))
    f.add_element(LinearSpring(f.nodes[2],f.nodes[3],k=3000))

    f.add_support(sid=0,nid=0,dof=[0,1,2],val=0.0)
    f.add_support(sid=0,nid=3,dof=[0,1,2],val=0.0)
    
    F = 5000
    
    pl1 = ff.PointLoad(sid=1,node=f.nodes[2],v=[F,0,0],f=1.0)
    f.add_load(pl1)
    
    f.add_loadcase(supp_id=0,load_id=1)
    
    
    f.nodal_dofs()
    f.linear_statics(lcase=1,support_method="REM")

    return f

def test_beam_and_spring():
    
    a = 3
    
    f = ff.FrameFEM()
    
    sect = ff.BeamSection(A=0.01,Iy=2e-4)
    
    f.add_section(sect)
    
    f.add_material(E=210e6,nu=.3,density=7850)
    
    f.add_node(0,0)
    f.add_node(a,0)
    f.add_node(2*a,0)
    f.add_node(2*a,-1e-3)
    
    f.add_element(EBBeam(f.nodes[0],f.nodes[1],sect,f.materials[0]))
    f.add_element(EBBeam(f.nodes[1],f.nodes[2],sect,f.materials[0]))
    f.add_element(LinearSpring(f.nodes[2],f.nodes[3],k=200))

    f.add_support(sid=0,nid=0,dof=[0,1,2],val=0.0)
    f.add_support(sid=0,nid=1,dof=[1],val=0.0)
    f.add_support(sid=0,nid=3,dof=[0,1],val=0.0)
    
    F = -50
    
    pl1 = ff.PointLoad(sid=1,node=f.nodes[2],v=[0,F,0],f=1.0)
    f.add_load(pl1)
    
    #ll1 = ff.LineLoad(sid=1,eid=f.elements[0],xloc=[0,1],qval=[-10,-10],direction='y')
    #f.add_load(ll1)    
    f.add_loadcase(supp_id=0,load_id=1)
    
    # Vapautetaan momentti
    #f.add_release(0,[5])
    #f.add_release(1,[2])
    
    f.nodal_dofs()
    f.linear_statics(lcase=1,support_method="REM")

    #f.draw()
    
    return f

def test3d_frame():
    
    L1 = 3000
    L2 = 6000
    h = 4500
    X = []
    X.append([0,0,0])
    X.append([0,L2,0])
    X.append([L1,L2,0])
    X.append([L1,0,0])
    X.append([0,0,h])
    X.append([0,L2,h])
    X.append([L1,L2,h])
    X.append([L1,0,h])
        
    col_sect = HEA(240)
    beam_sect = IPE(220)
    
    #col = ff.BeamSection(col_sect.A,col_sect.Iy)
    #beam = ff.BeamSection(beam_sect.A,beam_sect.Iy)
    
    f = ff.FrameFEM()
    
    for x in X:
        f.add_node(x[0],x[1],x[2])

    f.add_section(col_sect)
    f.add_section(beam_sect)
    
    f.add_material(col_sect.E,col_sect.material.nu,col_sect.density)

    for i in range(4):
        n1 = f.nodes[i]
        n2 = f.nodes[i+4]
        f.add_element(EBBeam3D(n1, n2, col_sect,f.materials[0]))

    for i in range(4,7):
        n1 = f.nodes[i]
        n2 = f.nodes[i+1]
        f.add_element(EBBeam3D(n1, n2, beam_sect,f.materials[0]))
        
        if i+1 == 7:
            n1 = f.nodes[i+1]
            n2 = f.nodes[4]
            f.add_element(EBBeam3D(n1, n2, beam_sect,f.materials[0]))
        
    for i in range(4):        
        f.add_support(sid=0,nid=i,dof=[0,1,2,3,4,5])
    
    F = -50e3
    
    pl1 = ff.PointLoad(sid=1,node=f.nodes[7],v=[0,0,F,0,0,0],f=1.0)
    f.add_load(pl1)
    
    #ll1 = ff.LineLoad(sid=1,eid=f.elements[0],xloc=[0,1],qval=[-10,-10],direction='y')
    #f.add_load(ll1)    
    f.add_loadcase(supp_id=0,load_id=1)
    
    # Vapautetaan momentti
    #f.add_release(0,[5])
    #f.add_release(1,[2])
    
    f.nodal_dofs()
    f.linear_statics(lcase=1,support_method="REM")
    
    f.draw()
    

    return f

def test3d_frame2():
    
    L1 = 3000
    L2 = 6000
    h = 4500
    X = []

    X.append([L2,L1,0])
    X.append([L2,L1,h])
    X.append([0,L1,h])    
    #X.append([L2,0,h])
    
    col_sect = HEA(240)
    beam_sect = IPE(220)
    
    #col = ff.BeamSection(col_sect.A,col_sect.Iy)
    #beam = ff.BeamSection(beam_sect.A,beam_sect.Iy)
    
    f = ff.FrameFEM()
    
    for x in X:
        f.add_node(x[0],x[1],x[2])

    f.add_section(col_sect)
    f.add_section(beam_sect)
    
    f.add_material(col_sect.E,col_sect.material.nu,col_sect.density)

    f.add_element(EBBeam3D(f.nodes[0], f.nodes[1], col_sect,f.materials[0]))
    f.add_element(EBBeam3D(f.nodes[2], f.nodes[1], beam_sect,f.materials[0]))
    #f.add_element(EBBeam3D(f.nodes[2], f.nodes[1], beam_sect,f.materials[0]))

          
    f.add_support(sid=0,nid=0,dof=[0,1,2,3,4,5])
    f.add_support(sid=0,nid=2,dof=[0,1,2,3,4,5])
    #f.add_support(sid=0,nid=3,dof=[0,1,2,3,4,5])
    
    F = -100e3
    
    pl1 = ff.PointLoad(sid=1,node=f.nodes[1],v=[0,0,F,0,0,0],f=1.0)
    f.add_load(pl1)
    
    #ll1 = ff.LineLoad(sid=1,eid=f.elements[0],xloc=[0,1],qval=[-10,-10],direction='y')
    #f.add_load(ll1)    
    f.add_loadcase(supp_id=0,load_id=1)
    
    # Vapautetaan momentti
    #f.add_release(0,[5])
    #f.add_release(1,[2])
    
    f.draw()
    
    f.nodal_dofs()
    f.linear_statics(lcase=1,support_method="REM")
    
    
    

    return f

def test3d_beam():
    
    L1 = 3000
    L2 = 6000
    h = 4500
    X = []
    X.append([0,0,0])
    X.append([0,0,h])
        
    col_sect = HEA(240)
    beam_sect = IPE(220)
    
    #col = ff.BeamSection(col_sect.A,col_sect.Iy)
    #beam = ff.BeamSection(beam_sect.A,beam_sect.Iy)
    
    f = ff.FrameFEM()
    

    for x in X:
        f.add_node(x[0],x[1],x[2])

    f.add_section(col_sect)
    #f.add_section(beam_sect)
    
    f.add_material(col_sect.E,col_sect.material.nu,col_sect.density)

    f.add_element(EBBeam3D(f.nodes[0], f.nodes[1], col_sect,f.materials[0]))

    f.add_support(sid=0,nid=0,dof=[0,1,2,3,4,5])
    
    F = 100e3
    
    pl1 = ff.PointLoad(sid=1,node=f.nodes[1],v=[0,0,-F,0,0,0],f=1.0)
    f.add_load(pl1)
    
    #ll1 = ff.LineLoad(sid=1,eid=f.elements[0],xloc=[0,1],qval=[-10,-10],direction='z',coords='local')
    #f.add_load(ll1)    
    f.add_loadcase(supp_id=0,load_id=1)
    
    # Vapautetaan momentti
    #f.add_release(0,[5])
    #f.add_release(1,[2])
    
    f.draw()
    
    f.nodal_dofs()
    f.linear_statics(lcase=1,support_method="REM")
    
    
    

    return f

if __name__ == '__main__':
    
    #pf, KG = simple_portal_fem(L=4000,H=4000,ne=4)
    
    #pf = simple_column(L=8000,ne=6)
    
    #f = test_hinge()

    #f = test_springs()
    
    #f = test_beam_and_spring()
    
    f = test3d_frame()
    #f = test3d_frame2()
    #f = test3d_beam()


# -*- coding: utf-8 -*-
# Copyright 2022 Kristo Mela
# This source code is licensed under the MIT license. See LICENSE in the repository root directory.
# Author(s): Kristo Mela
import ast
import os
import matplotlib.pyplot as plt
import plotly.graph_objects as go
import numpy as np
import pandas as pd

from metku.frame2d.frame2d import *
from matplotlib.animation import FuncAnimation

class ResultPlotter:

    def __init__(self, filename):

        self.filename = filename
        self.df = pd.read_csv(filename, sep=',')
        numeric_columns = ['f*', 'Iterations', 'FEM Analyses']
        self.df[numeric_columns] = self.df[numeric_columns].apply(
            pd.to_numeric)

    def iteration_gif(self, idx=0, scaler=0, interval=100, save=False):
        """
        Creates a gif animation of the objective function at each iteration
        :param  df: pandas dataframe
        :param interval: ms between each frame
        """
        fvals = self.df['Objective Values'].values[idx]
        fvals = ast.literal_eval(fvals)

        gvals = self.df['Constraint Values'].values[idx]
        gvals = ast.literal_eval(gvals)
        gvals = [max(gval) for gval in gvals]

        states = self.df['States'][idx]
        states = ast.literal_eval(states)

        fig, (ax3, ax1, ax2) = plt.subplots(3, 1,
                                            constrained_layout=True)
        title = ""
        cols = self.df.columns[17:]
        vals = self.df[cols].values[idx]
        for i in range(len(cols)):
            title += cols[i] + ": " +str(vals[i])+ "\n "
        fig.suptitle(title)

        if not scaler:
            scaler = int(len(fvals)/5)
        best_fval = [np.inf]

        def update(i):
            gval = gvals[i]
            num_vals = len(fvals)
            j = max(0, i - scaler)
            fval = fvals[i]
            if fval < best_fval[0] and gval < 0:
                best_fval[0] = fval
            # Objective value
            # Line chart
            ax1.clear()

            ax1.plot(np.arange(j, i), fvals[j:i])
            ax1_label = f'Iteration {i}/{num_vals} Best: {best_fval[0]:.2f}'
            ax1.set_xlabel(ax1_label)
            ax1.set_xlim(j, min(num_vals, i + scaler))
            ax1.set_ylim(min(fvals[j:]) - 100, max(fvals[j:]) + 100)

            # Constraint value
            # Bar chart
            if gval > 0:
                ax2.bar(i, gval, color='r')
            else:
                ax2.bar(i, gval, color='g')
            ax2.set_xlim(j, min(num_vals, i + scaler))
            ax2.set_ylim(min(gvals[j:]) - 0.05, max(gvals[j:]) + 0.05)
            ax2.set_xlabel("Max Constraint val: " + str(round(gval, 4)))

            # States
            # Matrix plot
            # TODO: What if problem is continuous?
            state = states[i]
            ax3.clear()


            one_hot = np.eye(64)[state]
            if gval >= 0:
                cmap = "Reds_r"
            else:
                cmap = "Greens_r"
            ax3.set_yticks(np.arange(0, len(state)))
            ax3.set_yticks([y - 0.5 for y in ax3.get_yticks()][1:])
            ax3.matshow(one_hot, cmap=cmap)
            ax3.grid(False)


            return ax1, ax2, ax3

        def init():
            ax1.clear()
            ax2.clear()
            ax3.clear()

        anim = FuncAnimation(fig,
                             update,
                             frames=len(fvals),
                             init_func=init,
                             interval=interval,
                             repeat_delay=500)
        if save:
            filename = self.filename[:-4] + "_idx_" + str(idx) + ".mp4"
            anim.save(filename)
            print(f"Animation saved as: {filename} to {os.getcwd()}")
        else:
            plt.show()

    def structure_gif(self, problem, idx=0, save=False):
        """
        Animates how states change during iteration
        :return:
        """
        states = self.df['States'][idx]
        states = ast.literal_eval(states)
        gvals = self.df['Constraint Values'].values[idx]
        gvals = ast.literal_eval(gvals)
        gvals = [max(gval) for gval in gvals]
        fvals = self.df['Objective Values'].values[idx]
        fvals = ast.literal_eval(fvals)
        best_f = [np.inf]
        best_x = [states[0]]
        best_g = [gvals[0]]

        fig, ax = plt.subplots()

        def update(i):
            label = f'Iteration {i}'
            state = states[i]
            gval = gvals[i]
            fval = fvals[i]
            if gval > 0:
                color = 'r'

            else:
                color = 'g'
                if fval < best_f[0]:
                    best_f[0] = fval
                    best_x[0] = state
                    best_g[0] = gval

            problem.substitute_variables(state)
            ax.clear()
            ax.set_xlim(-500, problem.structure.L + 500)
            if problem.name == "FifteenBarTruss":
                ax.set_ylim(-1000, 4000)
            if i == len(states) - 1:
                problem.substitute_variables(best_x[0])
                ax.set_xlabel(f"Objective: {best_f[0]:.2f} "
                              f" Max g: {best_g[0]:.4f}")
                color = 'g'

                for var in problem.vars:
                    for obj in var.target['objects']:
                        if isinstance(obj, FrameMember):
                            (x1, y1), (x2, y2) = obj.coordinates
                            ind = var.profiles.index(var.value)
                            ind = max(0.2, ind / 5)
                            ax.plot([x1, x2], [y1, y2],
                                    linewidth=ind, color=color)
            else:
                ax.set_xlabel(f"Iteration {i} / {len(fvals)} "
                              f"Objective: {fvals[i]:.2f} "
                              f" Max g: {gval:.4f}")

                for var in problem.vars:
                    for obj in var.target['objects'] :
                        if isinstance(obj, FrameMember):
                            (x1, y1), (x2, y2) = obj.coordinates
                            idx = var.profiles.index(var.value)
                            idx = max(0.2, idx/5)
                            ax.plot([x1, x2], [y1, y2],
                                    linewidth=idx, color=color)





            return ax

        anim = FuncAnimation(fig,
                             update,
                             frames=len(states),
                             interval=50,
                             repeat_delay=5000)


        if save:
            filename = self.filename[:-4] + "_idx_" + str(idx) + ".mp4"
            anim.save(filename)
            print(f"Animation saved as: {filename} to {os.getcwd()}")
        else:
            plt.show()


    def optimal_results(self):
        """
        Plots objective values as barchart
        :return:
        """
        # Values from csv
        obj_vals = self.df['f*'].values
        min_val = min(obj_vals)
        max_val = max(obj_vals)
        mean = np.mean(obj_vals)
        std = np.std(np.nan_to_num(obj_vals))
        feasible = self.df['Feasible'].values
        feasible_vals = obj_vals[feasible]
        if np.any(feasible_vals):
            min_feasible = min(feasible_vals)
        else:
            min_feasible = np.inf
        legend_values = [[], []]
        X = []
        for i in range(len(obj_vals)):
            if self.df['Feasible'][i]:
                color = 'tab:green'
                if np.isclose(obj_vals[i], min_feasible):
                    color = 'tab:blue'
            else:
                color = 'tab:red'
            plt.bar(i, obj_vals[i], color=color)
            X.append(i)
        plt.ylim(min_val - std , max_val + std)
        ticks = np.arange(np.round(min_val - std, -2) , max_val + std, 100)
        #plt.yticks(ticks)
        plt.xticks(np.arange(0, len(obj_vals)))
        plt.ylabel("Objective Value")
        plt.show()


    def table(self):
        """
        Plots a table of results
        :return:
        """
        columns = ['Feasible',
                   'f*',
                   'x*',
                   'x0',
                   'Iterations',
                   'FEM Analyses']

        columns.extend(self.df.columns[17:])

        print(self.df[columns].to_string())

        # rows = ['gamma']
        # gammas = self.df['gamma'].values
        # move_limits = self.df['move_limits'].values
        #
        # rows = []
        # for i in range(len(gammas)):
        #     gamma = gammas[i]
        #     move_limit = move_limits[i]
        #     rows.append(f'{i}: {move_limit} {gamma}')

        #
        # fig = go.Figure(data=[go.Table(
        #     header=dict(values=columns),
        #     cells=dict(values=self.df[columns].values.T))])
        # fig.show()

        # plt.table(
        #     cellText=self.df[columns].values,
        #     rowLabels=rows,
        #     colLabels=columns,
        #     loc='center'
        # )
        # plt.show()


if __name__ == '__main__':
    filename = "results/VNS_TenBarTruss_2019_08_28.csv"
    #filename = "benchmarks/MISLP_FiftyTwoBarTruss_2019_08_21.csv"

    plotter = ResultPlotter(filename)
    # plotter.optimal_results()
    # plotter.iteration_gif(idx=3, scaler=10, save=False)
    # plotter.table()
    # from metku.optimization.benchmarks import *
    # problem = TenBarTruss()
    # plotter.structure_gif(problem, idx=14, save=True)
    print(plotter.df[['Feasible', 'f*']].to_latex())
    """
    HYVIÄ ESIMERKKEJÄ
    
    VNS Oskillaatio kahden arvon välissä
    # filename = "results/VNS_FiftyTwoBarTruss_2019_08_23.csv"
    # plotter = ResultPlotter(filename)
    # plotter.iteration_gif(idx=8, scaler=25)
    
    VNS "kuolee": juuttuu linearisoidun alitehtävän ratkaisuun,
    joka on oikeasti epäkäypä
    # filename = "results/VNS_FiftyTwoBarTruss_2019_08_23.csv"
    # plotter = ResultPlotter(filename)
    # plotter.iteration_gif(idx=2, scaler=10)
    
    
    """


# Author(s): Kristo Mela
# This source code is licensed under the MIT license. See LICENSE in the repository root directory.
# Copyright 2022 Kristo Mela
# -*- coding: utf-8 -*-
import pygame
import numpy as np
from metku.frame2d.frame2d import *
from metku.truss2d import Truss2D
from metku.sections.steel.catalogue import *

PROFILES = list(rhs_profiles.keys())

GREEN = (0, 255, 0)
RED = (255, 0, 0)



frame = Frame2D(simple=[2,2, 1e3, 1e3], supports='fixed', num_elements=10)

for mem in frame.members.values():
    if mem.mtype == 'beam':
        frame.add(LineLoad(mem, [-1e3, -1e3], 'y'))

frame.add(PointLoad([0, 1e3], [2e5, 0, 0]))
frame.generate()
#frame.hinge_joints()
frame.calculate()


WIDTH = 500
HEIGHT = 500
pygame.init()

SCALE = frame.L / WIDTH * 2

HEIGHT = max(HEIGHT, int(frame.H / SCALE) + 100)

screen = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("Frame2D")

pressed = 0
RUN = True

def draw_frame():

    # Members
    for mem in frame.members.values():
        start, end = np.asarray(mem.coordinates) / SCALE
        start = np.array([3/2*frame.L/SCALE , HEIGHT - 30]) - start
        end = np.array([3/2*frame.L/SCALE , HEIGHT - 30]) - end
        pygame.draw.line(screen, (0, 0, 0), start, end,3)

        # Hinges
        if  mem.Sj1 <= 1e-4:
            size = 50 / SCALE
            pygame.draw.ellipse(screen,  (0,0,0), (start[0]-size/2, start[1] - size / 2, size, size))
            pygame.draw.ellipse(screen, (255, 255, 255), (start[0] - size / 4, start[1] - size / 4, size/2, size/2))

        if  mem.Sj2 <= 1e-4:
            size = 50 / SCALE
            pygame.draw.ellipse(screen, (0, 0, 0), (end[0] - size / 2, end[1] - size / 2, size, size))
            pygame.draw.ellipse(screen, (255, 255, 255), (end[0] - size / 4, end[1] - size / 4, size / 2, size / 2))


    # Supports
    for sup in frame.supports.values():
        rect_size = 60 / SCALE
        x, y = np.asarray(sup.coordinate) / SCALE
        x  = 3/2*frame.L/SCALE - x - rect_size/2
        y = HEIGHT - 30 - y
        rect = pygame.Rect(x, y, rect_size, rect_size)
        pygame.draw.rect(screen , (0,0,0), rect)


def draw_deflected(scl=1):

    for mem in frame.members.values():

        if max(mem.r[:-1]) < 1:
            color = GREEN
        else:
            color = RED

        x_vals = np.asarray([coord[0] for coord in mem.nodal_coordinates])
        y_vals = np.asarray([coord[1] for coord in mem.nodal_coordinates])


        x_disp_vals = np.asarray([disp[0] for disp in mem.nodal_displacements.values()])
        y_disp_vals = np.asarray([disp[1] for disp in mem.nodal_displacements.values()])


        x_vals -= x_disp_vals
        y_vals += y_disp_vals

        x_vals /= SCALE
        y_vals /= SCALE

        x_vals = 3/2*frame.L/SCALE - x_vals
        y_vals = HEIGHT - 30 - y_vals

        pointlist = []

        for x, y in zip(x_vals, y_vals):
            pointlist.append([int(x), int(y)])
        pygame.draw.lines(screen, color, False,  pointlist, 2)



def update_frame(i=[]):

    idx = len(i)

    if idx < len(PROFILES):
        for mem in frame.members.values():
            mem.profile = PROFILES[idx]
        frame.calculate()
        i.append(i)






while RUN:
    screen.fill((255, 255, 255))
    pygame.time.delay(300)
    if pressed:
        pressed = 0

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            RUN = False

    draw_frame()
    draw_deflected()
    update_frame()

    pygame.display.flip()

pygame.quit()



# -*- coding: utf-8 -*-
# Copyright 2022 Kristo Mela
# This source code is licensed under the MIT license. See LICENSE in the repository root directory.
# Author(s): Kristo Mela

"""
from workshop import BlastingCost

bc = BlastingCost(length=40,width=10)

bc.costs

print("Blasting")
for key, value in bc.costs.items():
    print("{0} cost = {1:4.2f} €/min".format(key,value))
    
bc.ProductiveTime(4000)

print(bc.cost())
"""

"""
tip_flow = 900000 # [mm3/min]

paints = {"alkyd": {"vol_solid":[0.48,0.45,0.45], "DFT":[80,40,40],"price":[2.8,3.6,3.6],"time":3,"TPP":0.0,"CP":0.0},
          "epoxy": {"vol_solid":[0.50,0.50,0.48], "DFT":[60,60,40],"price":[4.2,4.2,4.9],"time":14,"TPP":0.0,"CP":0.0},
          "polyurethane": {"vol_solid":[0.50,0.50], "DFT":[60,60],"price":[4.2,5.9],"time":27,"TPP":0.0,"CP":0.0},
          "acryl": {"vol_solid":[0.48,0.48,0.40], "DFT":[60,60,40],"price":[6.4,6.4,6.4],"time":7,"TPP":0.0,"CP":0.0},
          }

for paint, films in paints.items():
    print(paint)
    nfilms = len(films["vol_solid"])
    TPP = 0.0
    CP = 0.0
    for i in range(nfilms):
        tloss = 0.01+0.3*films["DFT"][i]*1e-3
        P0 = 1/films["vol_solid"][i]*(films["DFT"][i]*1e-3+tloss)
        TPP += P0
        CP += films["price"][i]*1e-6*P0

    TPP = TPP/tip_flow
    paints[paint]["TPP"] = TPP
    paints[paint]["CP"] = CP
"""

from metku.workshop import AssemblingCostWeld

ac = AssemblingCostWeld(length=15,width=5)
 